/*jslint nomen: true, plusplus: true*/
/*global define, window*/

define("de_epages/mediagallery/filemanager/list-view", [
 "backbone",
 "underscore",
 "jquery",
 "$tmpl!./list-view",
 "./folder-model",
 "./file-collection",
 "./folder-collection",
 "./file-view",
 "./folder-view",
 "$dict!../dictionary",

 "jquery/endlessscroll",
 "jquery/ui/selectable"
], function (Backbone, _, $, template, FolderModel, FileCollection, FolderCollection, FileView, FolderView, mediagalleryDict) {
    "use strict";

    var tMediaGallery = mediagalleryDict.translate("MediaGallery"),
        tBackgroundImages = mediagalleryDict.translate("BackgroundImages"),
        tHeaderImages = mediagalleryDict.translate("HeaderImages"),
        tImageDb = mediagalleryDict.translate("ImageDb"),
        tReverseSortOrder = mediagalleryDict.translate("ReverseSortOrder"),
        tNewFolderName = mediagalleryDict.translate("NewFolder"),

        view = Backbone.View.extend({
            tagName: "div",
            className: "ep-mediagallery-listView",
            template: template,
            fileCssClass: "ep-mediagallery-fileContainer",
            folderCssClass: "ep-mediagallery-folderContainer",

            events: {
                "click .changeOrder": "changeOrder"
            },

            initialize: function () {
                this.reverse = false;
            },

            render: function () {
                var self = this,
                    name = self.model.get("data"),
                    maxRender = self.options.maxRender,
                    tName,
                    tmpl,
                    l,
                    i;

                switch (name) {
//#JSCOVERAGE_IF false
                case "MediaGallery":
                    tName = tMediaGallery;
                    break;
                case "HeaderImages":
                    tName = tHeaderImages;
                    break;
                case "BackgroundImages":
                    tName = tBackgroundImages;
                    break;
                case "ImageDB":
                    tName = tImageDb;
                    break;
//#JSCOVERAGE_ENDIF
                default:
                    tName = self.model.get("metadata").displayname || name;
                    break;
                }

                tmpl = self.template({
                    name: tName,
                    reverse: self.reverse,
                    tReverseSortOrder: tReverseSortOrder
                });

                $.extend(self, $.tmplItem(tmpl).elements);

                self.$el.html(tmpl.children());

                // if files isn't already a file collection, save it as that
                if (_.isArray(self.model.get("files"))) {
                    self.model.set({
                        files: new FileCollection(self.model.get("files"))
                    });
                }
                self.files = self.model.get("files");

                // if folders isn't already a folder collection, save it as that
                if (_.isArray(self.model.get("children"))) {
                    self.model.set({
                        children: new FolderCollection(self.model.get("children"))
                    });
                }
                self.folders = self.model.get("children");

                self.foldersLength = self.folders.models.length;
                self.filesLength = self.files.models.length;
                self.currentFolder = 0;
                self.currentFile = 0;

                // add file and folder views in the selected order
                // handling for normal sort order
                if (!self.reverse) {
                    for (i = 0; i < self.foldersLength && i < maxRender; i++) {
                        self.addFolderView(i);
                    }
                    self.currentFolder = i;
                    if (self.foldersLength <= maxRender) {
                        for (i = 0; i < self.filesLength && i < maxRender; i++) {
                            self.addFileView(i);
                        }
                        self.currentFile = i;
                    }
                } else {
                    // handling for reversed sort order
                    for (i = self.foldersLength - 1; i > self.foldersLength - maxRender && i >= 0; i--) {
                        self.addFolderView(i);
                    }
                    self.currentFolder = i;
                    if (self.foldersLength <= maxRender) {
                        for (i = self.filesLength - 1; i > self.filesLength - maxRender && i >= 0; i--) {
                            self.addFileView(i);
                        }
                        self.currentFile = i;
                    }
                }
                self.setSelectable();

                // clear interval from endless scroll plugin because it never stops
                window.clearInterval(self.myInterval);

//#JSCOVERAGE_IF false
                // use endless scroll plugin to add next dom nodes by scrolling
                self.myInterval = self.scrollContainer.endlessScroll({
                    fireOnce: true,
                    ceaseFireOnEmpty: false,
                    inflowPixels: 0,
                    loader: '<div style="display:none;"></div>',
                    insertBefore: 'div.ep-mediagallery-listViewFooter',
                    insertAfter: 'div.ep-mediagallery-listViewFooter',
                    callback: function (fireSequence, pageSequence, scrollDirection) {
                        // only add new items when scrolling down
                        if (scrollDirection === 'next') {
                            // handling for normal sort order
                            if (!self.reverse) {
                                if (self.foldersLength && self.currentFolder < self.foldersLength) {
                                    for (i = self.currentFolder; i < self.foldersLength && i < self.currentFolder + maxRender; i++) {
                                        self.addFolderView(i);
                                    }
                                    self.currentFolder = i;
                                }
                                if (self.filesLength && self.currentFolder >= self.foldersLength && self.currentFile < self.filesLength) {
                                    for (i = self.currentFile; i < self.filesLength && i < self.currentFile + maxRender; i++) {
                                        self.addFileView(i);
                                    }
                                    self.currentFile = i;
                                }
                            } else {
                                // handling for reversed sort order
                                if (self.foldersLength && self.currentFolder >= 0) {
                                    for (i = self.currentFolder; i > self.currentFolder - maxRender && i >= 0; i--) {
                                        self.addFolderView(i);
                                    }
                                    self.currentFolder = i;
                                }
                                if (self.filesLength && self.currentFolder <= 0 && self.currentFile >= 0) {
                                    for (i = self.currentFile; i > self.currentFile - maxRender && i >= 0; i--) {
                                        self.addFileView(i);
                                    }
                                    self.currentFile = i;
                                }
                            }

                        	if (self.options.multiSelect) {
                            	self.list.selectable('destroy');
                        	}
                            self.setSelectable();
                        }
                    },
                    ceaseFire: function (fireSequence, pageSequence, scrollDirection) {
                        if (!self.reverse) {
                            if (self.foldersLength <= self.currentFolder && self.filesLength <= self.currentFile) {
                                return true;
                            }
                        } else {
                            if (self.currentFolder <= 0 && self.currentFile <= 0) {
                                return true;
                            }
                        }
                    }
                });
//#JSCOVERAGE_ENDIF

                return self;
            },

            // set items selectable
            setSelectable: function () {
                var self = this;

                if (self.options.multiSelect) {
                    self.list.selectable({
                        filter: 'li',
                        unselecting: function (event, ui) {
                            self.$("input").trigger("blur");
                        },
                        selected: $.proxy(self.selectableselected, self)
                    });
                }
                else{
                    self.list.find('li').on('click', function () {
                        self.list.find('li.ui-selected').removeClass('ui-selected');
                        $(this).addClass('ui-selected');
                        self.selectableselected();
                    });
                }
            },

            addFileView: function (i) {
                var item = this.files.models[i],
                    fileView;

                item.set({
                    metadata: {
                        type: this.model.get("metadata").type
                    },
                    id: item.get("directory") + item.get("path"),
                    list: this.$el
                });

                fileView = new FileView({
                    model: item
                });

                this.list.append(fileView.render().el);
            },

            addFolderView: function (i) {
                var item = this.folders.models[i],
                    folderView;

                item.set({
                    id: item.get("metadata").type + item.get("metadata").path
                });

                folderView = new FolderView({
                    model: item
                });

                folderView.on("openFolder", this.triggerOpenFolder, this).on("renameTreeNode", this.renameTreeNode, this);

                this.list.append(folderView.render().el);
            },

            // change display order of file and folder views
            changeOrder: function () {
                this.reverse = this.reverse ? false : true;
                this.render();
            },

            triggerOpenFolder: function (data) {
                this.trigger("openFolder", data);
            },

            renameTreeNode: function (data) {
                this.trigger("renameTreeNode", data);
            },

            getSelectedElements: function () {
                var models = [];

                this.selected = this.$("li.ui-selected");

                _.each(this.selected, function (item) {
                    var $item = $(item),
                        itemID = $item.data("itemID");

                    if ($item.hasClass(this.fileCssClass)) {
                        models.push(this.files.where({
                            id: itemID
                        })[0]);
                    } else if ($item.hasClass(this.folderCssClass)) {
                        models.push(this.folders.where({
                            id: itemID
                        })[0]);
                    }
                }, this);

                return models;
            },

            // collect selected elements and trigger selected event
            selectableselected: function () {
                this.trigger("selectableselected", this.getSelectedElements());
            },

            // create new folder model/view and save it
            createNewFolder: function () {
                var lastFolder = this.$(".ep-mediagallery-folderContainer:last"),
                    newFolder = new FolderModel(),
                    newFolderView = new FolderView({
                        model: newFolder
                    });

                this.folders.add(newFolder);

                newFolder.save({
                    data: tNewFolderName
                }, {
                    method: "createDirectory",
                    directory: this.model.get("metadata").path,
                    view: this
                });

                if (lastFolder.length) {
                    lastFolder.after(newFolderView.render().el);
                } else {
                    this.list.prepend(newFolderView.render().el);
                }

                newFolderView.on("openFolder", this.triggerOpenFolder, this).on("renameTreeNode", this.renameTreeNode, this);
            },

            // delete the selected items
            deleteItems: function () {
                _.each(this.selected, function (item) {
                    var $item = $(item),
                        itemID = $item.data("itemID"),
                        model;

                    if ($item.hasClass(this.fileCssClass)) {
                        model = this.files.where({
                            id: itemID
                        })[0];
                        this.files.remove(model);
                        model.destroy({
                            method: "deleteFile",
                            view: this
                        });
                    }
                    if ($item.hasClass(this.folderCssClass)) {
                        model = this.folders.where({
                            id: itemID
                        })[0];
                        this.folders.remove(model);
                        model.destroy({
                            method: "deleteDirectory",
                            view: this
                        });
                    }
                }, this);

                this.selected.remove();
            }
        });

    return view;
});